/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb der Analog-Tastatur PT-ATT! Anzeige der     */
/*                          bettigten Taste auf HyperTerm!                     */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     Zusatz Hardware:     PT - Analog Tastatur (PT-ATT)                       */
/*                                                                              */
/*                          Analogeingang der Tastatur P1.0:                    */
/*                              Anschluss: X12 PT-FS   <->   X1.1 PT - ATT)     */
/*                          Summer P3.7:                                        */
/*                              Anschluss: X9.3 PT-FS  <->   X1.5 PT - ATT)     */
/*                          Spannungsversorgung der PT-ATT, GND und VCC:        */
/*                              Anschluss: X9.1 PT-FS  <->   X1.6 PT - ATT)     */
/*                              Anschluss: X9.4 PT-FS  <->   X1.2 PT - ATT)     */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     14.01.2010                                          */
/*                                                                              */
/*     Datei:               ANA_TAST.c                                          */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

/*** Definition der globalen Variablen  ***/
unsigned char bit SUMMER    @   0x91;   // Portpin P1.1, SUMMER

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*  Funktion void set_cursor(unsigned char z, unsigned char s) positioniert den */
/*  Terminal Cursor auf die angegebene Position:                                */
/*  Parameter z := Zeile, Parameter s := Spalte                                 */
/********************************************************************************/

void set_cursor(unsigned char z, unsigned char s)
  {
    printf("\x1b\x59%c%c",(32+z),(32+s));
  }  

/********************************************************************************/
/*  Die Funktion unsigned int mw_erf10(unsigned char kanal) dient zum Betrieb   */
/*  des ON-Chip A/D-Wandlers des AT89C51CC03. Der Funktion wird der Eingangs-   */
/*  kanal bergeben und sie liefert ein 10-Bit Wandlungsergebnis als Funktions- */
/*  wert zurck.                                                                */
/*  Wichtig:        Diese Funktion setzt voraus, das die Grundinitialisierung   */
/*                  des ON-Chip A/D-Wandlers ausserhalb der Funktion, also      */
/*                  z. B. im Hauptprogramm erfolgt. Zur Grundinitialisierung    */
/*                  muss der A/D-Wandler eingeschaltet werden, also das         */
/*                  ADEN-Bit im ADCON SFR gesetz werden ( ADCON = 0x20) und     */
/*                  der zu wandelnde Eingangskanal im SFR ADCF als Analog-      */
/*                  eingang definiert werden (z. B. ADCF = 0xff ->              */
/*                  gesamter Port 1 als Analogeingang definiert!                */
/*  Parameter kanal := Auswahl des A/D-Wandler Eingangskanals (0..7)            */
/********************************************************************************/

unsigned int mw_erf10(unsigned char kanal)
  {      
    unsigned int erg;                       // Variable fr 10-Bit Wandlungsergebnis
    ADCON = (ADCON & 0xe8);                 // Kanalauswahl und Wandlungsende-Bit rcksetzen!
    ADCON = (ADCON | (0x08+kanal));         // Kanalauswahl und Start der einmaligen Wandlung!
    while((ADCON & 0x10) == 0);             // Warte bis A/D-Wandlung beendet ist!
    erg = ((ADDH << 2) + (ADDL & 0x03));    // Ergebnisregister auslesen u. zusammenbauen!
    return(erg);
  }

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main(void)
  {
    unsigned int wert;                                  // Hilfsvariable 10-Bit Wandlungswert
    unsigned char taste;                                // Hilfsvariable bettigte Taste
    seri_init();                                        // Initialisierung der ser. Schnittstelle
    clear_screen();                                     // Bildschirm lschen

    printf(" Betrieb der Analog-Tastatur! \n");         // Textmeldungen und Bildschirmmaske auf Hyperterm
    printf("==============================\n\n");
    printf("   Programm laeuft ... \n\n\n\n\n");
    printf("   Folgende Taste ist betaetigt: ");

    ADCF = 0x01;                                        // Port P1.0 als Analogeingang!
    ADCON = 0x20;                                       // ON-Chip A/D-Wandler eingeschaltet!
    while(1)                                            // Endlosschleife!      
      {
        wert = mw_erf10(0);                                 // 10-Bit Messwert von Port P1.0 ermitteln
        set_cursor(8,34);

        if (wert<42) taste = 1;                             // Vergleich, welche Taste bettigt wurde!
        if ((wert>=42)&(wert<130)) taste = 2;
        if ((wert>=130)&(wert<213)) taste = 3;
        if ((wert>=213)&(wert<293)) taste = 4;
        if ((wert>=293)&(wert<381)) taste = 5;
        if ((wert>=381)&(wert<473)) taste = 6;
        if ((wert>=473)&(wert<553)) taste = 7;
        if ((wert>=553)&(wert<642)) taste = 8;
        if ((wert>=642)&(wert<726)) taste = 9;
        if ((wert>=726)&(wert<810)) taste = 10;
        if ((wert>=810)&(wert<897)) taste = 11;
        if ((wert>=897)&(wert<981)) taste = 12;
        if (wert>=981) taste = 0;                           // Keine Taste bettigt!

        if(taste)                                           // Wenn eine Taste bettigt ist,
          {                                                 // Piepton ausgeben und
            SUMMER = 0;                                     // bettigte Taste auf HyperTerm
            printf("%2u ",taste);                           // darstellen!
            _wait_ms(100);
            SUMMER = 1;
            _wait_ms(300);
          }
        else printf("--");
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
