/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb des ser. D/A-Wander Moduls PT - MAX522,     */
/*                          Ausgabe von zwei Sgezahnspannungen!                */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     Zusatz Hardware:     PT - MAX522                                         */
/*                                                                              */
/*                          Verbindung per 6-poliger MiniDIN-Leitung            */
/*                              Anschluss: X7 PT-FS  <->  X1 PT - MAX522        */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     18.01.2010                                          */
/*                                                                              */
/*     Datei:               DA_WANDLER.c                                        */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

/*** Definition der globalen Variablen  ***/
unsigned char bit DIN       @   0xb3;   // Portpin P3.3, SPI-Bus: serielle Datenleitung
unsigned char bit CLK       @   0xb2;   // Portpin P3.2, SPI-Bus: serielle Taktleitung
unsigned char bit CS        @   0xb4;   // Portpin P3.4, SPI-Bus: serielle Freigabeleitung

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*  Funktion void set_cursor(unsigned char z, unsigned char s) positioniert den */
/*  Terminal Cursor auf die angegebene Position:                                */
/*  Parameter z := Zeile, Parameter s := Spalte                                 */
/********************************************************************************/

void set_cursor(unsigned char z, unsigned char s)
  {
    printf("\x1b\x59%c%c",(32+z),(32+s));
  }  

/********************************************************************************/
/*  Die Funktion void send_spi_2(unsigned char adresse, unsigned char daten)    */
/*  sendet zwei Byte seriell vom C zur externen Peripherie-Einheit aus!        */
/*                                                                              */
/*  Die Parameter adresse und daten sind die auszusendenden Bytes,              */
/*  das MSB wird jeweils zuerst gesendet!                                       */
/*  Die Datenbernahme der externen Peripherieeinheit erfolgt bei ansteigender  */
/*  Taktflanke, das Freigabesignal ist aktiv LOW!                               */
/*                                                                              */
/*  Portleitungen:      P3.3 := Serielle Datenleitung (DIN)                     */
/*                      P3.2 := Serielle Taktleitung  (CLK)                     */
/*                      P3.4 := Freigabeleitung       (CS)                      */
/********************************************************************************/

void send_spi_2(unsigned char adresse, unsigned char daten)
  {
    unsigned char teiler, i;    // Hilfsvariablen zum bitweisen Austakten
    CS = 0;                     // Beginn der seriellen Datenbertragung
    teiler = 128;               // Bitmaske ( 1000 0000 ) fr das MSB
    for (i=1; i<9; i++)         // Schleife zum Austakten der 8 Adressbits
      {
        if(adresse & teiler) DIN = 1;   // DIN - Leitung bekommt den Logikpegel
        else DIN = 0;                   // des ausmaskierte, auszutaktenden Bits
        CLK = 1;                        // Takt ausgeben
        CLK = 0;
        teiler /= 2;                    // Bitmaske fr nchstes Adressbit setzen
      }
    teiler = 128;               // Bitmaske ( 1000 0000 ) fr das MSB
    for (i=1; i<9; i++)         // Schleife zum Austakten der 8 Datenbits
      {
        if(daten & teiler) DIN = 1;     // DIN - Leitung bekommt den Logikpegel
        else DIN = 0;                   // des ausmaskierte, auszutaktenden Bits
        CLK = 1;                        // Takt ausgeben
        CLK = 0;
        teiler /= 2;                    // Bitmaske fr nchstes Datenbit setzen
      }
    CS = 1;                   // Ende der seriellen Datenbertragung
  }


/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main (void)                
  {
    unsigned char i;            // Variable zur Berechnung des D/A-Wandler Ergebnisses
    
    seri_init();                                        // Initialisierung der ser. Schnittstelle
    clear_screen();                                     // Bildschirm lschen

    printf(" Betrieb des D/A-Wandlers MAX522!\n");      // Textmeldungen auf Hyperterm
    printf("==================================\n\n");
    printf("   Bitte Oszilloskop anschliessen, es werden\n");
    printf("   zwei Saegezahnspannungen ausgegeben!\n\n");
    printf("   Programm laeuft ... \n\n\n\n\n");

    CS = 1;                                             // Grundzustand auf dem SPI-Bus einstellen!
    CLK = 0;
    while(1)                                            // Endlosschleife!      
      {   
        for(i=0; i<255; i++)
          {
            send_spi_2(0x21,i);
            send_spi_2(0x22,(255-i));
          }
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
