/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb des I2C-Bus Frequenz Synthesizer-Moduls     */
/*                          PT - DS1085!                                        */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     Zusatz Hardware:     PT - DS1085                                         */
/*                                                                              */
/*                          Verbindung per 6-poliger MiniDIN-Leitung            */
/*                              Anschluss: X6 PT-FS  <->  X1 / X2 PT - DS1085   */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     15.01.2010                                          */
/*                                                                              */
/*     Datei:               FREQ_SYNTH.c                                        */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

#define PCF8574A_ADRESS  0x70           // Adresse des I2C-Bus Expander

/*** Definition der globalen Variablen  ***/
unsigned char bit SDA       @   0xc0;   // Portpin P4.0, I2C-Bus: serielle Datenleitung SDA
unsigned char bit SCL       @   0xc1;   // Portpin P4.1, I2C-Bus: serielle Taktleitung  SCL

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*  Funktion void set_cursor(unsigned char z, unsigned char s) positioniert den */
/*  Terminal Cursor auf die angegebene Position:                                */
/*  Parameter z := Zeile, Parameter s := Spalte                                 */
/********************************************************************************/

void set_cursor(unsigned char z, unsigned char s)
  {
    printf("\x1b\x59%c%c",(32+z),(32+s));
  }  

/********************************************************************************/
/*  Die Funktion void I2C_Start (void) erzeugt die (Repeated) Start-Bedingung   */
/*  des I2C-Busses!                                                             */
/********************************************************************************/

void I2C_Start (void)               
  {
    SDA = 1;
    SCL = 1;
    SDA = 0;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Stop (void) erzeugt die Stopp-Bedingung               */
/*  des I2C-Busses!                                                             */
/********************************************************************************/

void I2C_Stop (void)               
  {
    SDA = 0;
    SCL = 1;
    SDA = 1;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Master_Ack (void) sendet das Acknowledgement Signal   */
/*  des Masters ber den I2C-Bus aus!                                           */
/********************************************************************************/

void I2C_Master_Ack (void)          
  {
    SDA = 0;
    SCL = 1;
    SCL = 0;
    SDA = 1;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Master_NAck (void) sendet das Not Acknowledgement     */
/*  Signal des Masters ber den I2C-Bus aus!                                    */
/********************************************************************************/

void I2C_Master_NAck (void)         
  {
    SDA = 1;
    SCL = 1;
    SCL = 0;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Sende(unsigned char Daten) sendet den Parameter Daten */
/*  seriell ber den I2C-Bus aus! Anschlieend wird das ACK/NACK-Signal vom     */
/*  Slave empfangen und ausgewertet!                                            */
/********************************************************************************/

void I2C_Sende(unsigned char Daten) 
  {
    unsigned char Maske=128;        // Hilfsvariable zum ausmaskieren des Sendebytes!
    unsigned char i;                // Schleifenvariable
    SCL = 0;                        // Serielle Taktleitung auf LOW
    for(i=0; i<8; i++)
      {
        if(Maske & Daten) SDA = 1;      // Wenn auszutaktendes Bit HIGH ist, SDA  
        else SDA = 0;                   // auf log. 1, ansonsten auf log. 0 setzen!
        SCL = 1;                        // Takt ausgeben und Bitmaske fr den
        Maske = Maske / 2;              // nchsten Schleifendurchlauf vorbereiten!
        SCL=0;
      }
    SDA = 1;                            // Ser. Datenleitung auf Eingang schalten!
    SCL = 1;                            // 9. Takt fr Acknowledgement des DS1621!
    if(SDA)                             // Wenn SDA log. 1, dann Fehlermeldung,
      {
        printf("\n\n\n Datenuebertragungsfehler:");
        printf("\n Kein Acknowledgement vom Slave, Programm angehalten !");
        while(1);
      }
    SCL = 0;                            // ansonsten weiter!
  }

/********************************************************************************/
/*  Die Funktion void I2C_send_1(unsigned char CB, unsigned char cmd,           */
/*  unsigned char datum) sendet die Parameter CB (= Control Byte), cmd          */
/*  (= Commando Wort) und datum (= Datenbyte) seriell ber den I2C-Bus aus!     */
/********************************************************************************/

void I2C_send_1(unsigned char CB, unsigned char cmd, unsigned char datum)
  {    
    // Senden eines Bytes an den Salve
    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB);              // Sende Adresse des Slaves
    I2C_Sende(cmd);             // Sende Befehlswort an Salve
    I2C_Sende(datum);           // Sende Byte an Slave
    I2C_Stop();                 // I2C-Bus Stopp Bedingung
  }

/********************************************************************************/
/*  Die Funktion void I2C_send_1(unsigned char CB, unsigned char cmd,           */
/*  unsigned char dat1, unsigned char dat2) sendet die Parameter                */
/*  CB (= Control Byte), cmd (= Commando Wort), dat1 (= Datenbyte1) und         */
/*  dat2 (= Datenbyte2) seriell ber den I2C-Bus aus!                           */
/********************************************************************************/

void I2C_send_2(unsigned char CB, unsigned char cmd, unsigned char dat1, unsigned char dat2)
  {    
    // Senden der beiden Bytes an den Salve
    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB);              // Sende Adresse des Slaves
    I2C_Sende(cmd);             // Sende Befehlswort an Salve
    I2C_Sende(dat1);            // Sende 1. Byte an Slave
    I2C_Sende(dat2);            // Sende 2. Byte an Slave
    I2C_Stop();                 // I2C-Bus Stopp Bedingung
  }

/********************************************************************************/
/*  Die Funktion unsigned char I2C_Empf(void) empfngt 1 Datenyte seriell ber  */
/*  I2C-Bus und liefert dieses als Rckgabewert der Funktion!                   */
/********************************************************************************/

unsigned char I2C_Empf(void)    
  {
    unsigned char Ergebnis=0;   // Variable fr das zu empfangene Datenbyte!
    unsigned char Wert=128;     // Hilfsvariable zum aufaddieren des Ergebnisses!
    unsigned char i;            // Schleifenvariable
    SCL = 0;                    // Serielle Taktleitung auf LOW
    SDA = 1;                    // Serielle Datenleitung auf Eingang schalten!
    for(i=0; i<8; i++)
      {
        SCL = 1;                // Takt ausgeben, anschlieend Zustand auf SDA 
        if(SDA) Ergebnis = Ergebnis + Wert; // prfen und wenn log. 1 aufaddieren!
        SCL=0;
        Wert = Wert / 2;        // Aufzuaddierenden Wert fr nchsten 
      }                         // Schleifendurchlauf vorbereiten!
    SDA = 1;                    // Ser. Datenleitung auf Eingang schalten!
    return(Ergebnis);           // Funktionsergebnis zurckgeben!
  }

/********************************************************************************/
/*  Die Funktion unsigned int I2C_empf_2(unsigned char CB, unsigned char cmd)   */
/*  sendet die Parameter CB (= Control Byte) und cmd (= Commando Wort) seriell  */
/*  ber den I2C-Bus aus! Anschlieend werden 2 Datenbyte empfangen und als     */
/*  Rckgabewert der Funktion zur Verfgung gestellt!                           */
/********************************************************************************/

unsigned int I2C_empf_2(unsigned char CB, unsigned char cmd)
  {
    unsigned char HIGH;         // High-Byte
    unsigned char LOW;          // Low-Byte

    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB);              // Sende Adresse des Slaves: Schreiben
    I2C_Sende(cmd);             // Sende Befehlswort an Slave

    I2C_Start();                // I2C-Bus Repeated Start Bedingung
    I2C_Sende(CB+1);            // Sende Adresse des Slaves: Lesen

    HIGH = I2C_Empf();          // Einlesen des HIGH-Bytes aus dem Slave
    I2C_Master_Ack();           // Acknowlegdement Signal vom CC03er

    LOW = I2C_Empf();           // Einlesen des LOW-Bytes aus dem Slave
    I2C_Master_NAck();          // NOT Acknowlegdement Signal vom CC03er

    I2C_Stop();                 // I2C-Bus Stopp Bedingung

    return(HIGH*256 + LOW);     // Rckgabe des zusammengesetzten Wertes
  }

/********************************************************************************************/
/*** Start des Hauptprogramms ***************************************************************/
/********************************************************************************************/

void main (void)
  {
    unsigned char wahl;             // Hilfsvariable Frequenzauswahl 

    unsigned char OS;               // Hilfsvariable zur Zwischenspeicherung der Registerinhalte
    unsigned int zw;
    unsigned char zw1, zw2;
    
    signed char offset;             // Hilfsvariable zur Frequenzeinstellung
    unsigned int DAC;
    unsigned char Vorteiler;
    unsigned int N;
   
    seri_init();                                            // Initialisierung der ser. Schnittstelle
    while(1)                                                // Endlosschleife!      
      {
        clear_screen();                                         // Bildschirm lschen

        printf(" Betrieb des Frequenz Synthesizers!\n");        // Textmeldungen auf Hyperterm
        printf("====================================\n\n");
        printf("   Programm laeuft ... \n\n\n\n\n");
        printf("   Bitte geben Sie die Ausgangsfrequenz an:\n\n");
        printf("               5 kHz (1)           10 kHz (2)              15,5 kHz (3)\n");
        printf("             100 kHz (4)          248 kHz (5)               395 kHz (6)\n");
        printf("               1 MHz (7)        8,567 MHz (8)              24,7 MHz (9)\n");
        printf("            33,3 MHz (a)        65,78 MHz (b)               100 MHz (c)\n");
        printf("             133 MHz (d)         145  MHz (e)     Minimale Frequenz (f)\n");
        printf("   Maximale Frequenz (g)\n\n");
    
        printf("   Ihre Auswahl: ");
    
        wahl = getchar();                                   
        putchar(wahl);
    
        switch (wahl)
          {
            case '1' :  offset = -6;        DAC = 52;       // Werte fr 5 kHz
                        Vorteiler = 8;      N = 1025;       
                        printf("   Achtung, Frequenzschlauch (66 MHz - 133 MHz) verlassen!");
                        break;
            case '2' :  offset = -2;        DAC = 40;       // Werte fr 10 kHz
                        Vorteiler = 8;      N = 825;        break;
            case '3' :  offset = -1;        DAC = 80;       // Werte fr 15,5 kHz
                        Vorteiler = 8;      N = 600;        break;
            case '4' :  offset = -1;        DAC = 0;        // Werte fr 100 kHz
                        Vorteiler = 8;      N = 88;         break;
            case '5' :  offset = -1;        DAC = 80;       // Werte fr 248 kHz
                        Vorteiler = 2;      N = 150;        break;
            case '6' :  offset = -2;        DAC = 300;      // Werte fr 395 kHz
                        Vorteiler = 2;      N = 100;        break;
            case '7' :  offset = 3;         DAC = 0;        // Werte fr 1 MHz
                        Vorteiler = 8;      N = 12;         break;
            case '8' :  offset = 1;         DAC = 49;        // Werte fr 8,567 MHz
                        Vorteiler = 2;      N = 5;         
                        printf("   Achtung, exakter Wert: 8,565 MHz!");       
                        break;
            case '9' :  offset = 3;         DAC = 56;       // Werte fr 24,7 MHz
                        Vorteiler = 2;      N = 2;          break;
            case 'a' :  offset = 3;         DAC = 78;       // Werte fr 33,3 MHz
                        Vorteiler = 1;      N = 3;          break;
            case 'b' :  offset = 5;         DAC = 455;      // Werte fr 65,78 MHz
                        Vorteiler = 1;      N = 2;          
                        printf("   Achtung, exakter Wert: 65,775 MHz!");       
                        break;
            case 'c' :  offset = 3;         DAC = 80;       // Werte fr 100 MHz
                        Vorteiler = 1;      N = 1;          break;
            case 'd' :  offset = 3;         DAC = 740;      // Werte fr 133 MHz
                        Vorteiler = 1;      N = 1;          break;
            case 'e' :  offset = 3;         DAC = 980;      // Werte fr 145 MHz
                        Vorteiler = 1;      N = 1;          
                        printf("   Achtung, Frequenzschlauch (66 MHz - 133 MHz) verlassen!");
                        break;
            case 'f' :  offset = -6;        DAC = 0;        // Werte fr Minimale Frequenz
                        Vorteiler = 8;      N = 1025;      
                        printf("   Achtung, Frequenzschlauch (66 MHz - 133 MHz) verlassen!");
                        printf("\n   f := 4,683 kHz");
                        break;
            case 'g' :  offset = 6;         DAC = 1023;     // Werte fr Maximale Frequenz
                        Vorteiler = 1;      N = 1;         
                        printf("   Achtung, Frequenzschlauch (66 MHz - 133 MHz) verlassen!");
                        printf("\n   f := 166,4 MHz");
                        break;
          }
        
        zw = I2C_empf_2(0xb0,0x37);                 // OFFSET-Wert aus Range-Register auslesen, 
        OS = zw >> 11;                              // zurecht schieben, gewhlten Offset Wert
        I2C_send_1(0xb0,0x0e,OS+offset);            // aufaddieren und zurck schreiben!
        _wait_ms(10);                               // 10ms Abspeicherzeit, siehe Datenblatt
       
        zw = DAC << 6;                              // Gewhlten DAC-Wert zurecht schieben,
        zw1 = zw /256;                              // in HIGH-Byte und LOW-Byte aufteilen
        zw2 = zw -(zw1*256);                        // und Register einschreiben!
        I2C_send_2(0xb0,0x08,zw1,zw2);
        _wait_ms(10);                               // 10ms Abspeicherzeit, siehe Datenblatt
    
        switch (Vorteiler)                          // MUX Wort nach gewhlten Vorteiler schreiben
          {
            case 1 :    if (N > 1) I2C_send_2(0xb0,0x02,0x1e,0x00);    
                        else I2C_send_2(0xb0,0x02,0x1e,0x40); 
                        break;
            case 2 :    if (N > 1) I2C_send_2(0xb0,0x02,0x1e,0x80);   
                        else I2C_send_2(0xb0,0x02,0x1e,0xc0); 
                        break;
            case 4 :    if (N > 1) I2C_send_2(0xb0,0x02,0x1f,0x00);   
                        else I2C_send_2(0xb0,0x02,0x1e,0x40); 
                        break;
            case 8 :    if (N > 1) I2C_send_2(0xb0,0x02,0x1f,0x80);    
                        else I2C_send_2(0xb0,0x02,0x1e,0xc0); 
                        break;
          }
        _wait_ms(10);                               // 10ms Abspeicherzeit, siehe Datenblatt
        
        zw = (N-2) << 6;                            // Gewhlten DIV-Wert aufbereiten (N-2),
        zw1 = zw / 256;                             // zurecht schieben, in HIGH-Byte und 
        zw2 = zw - (zw1*256);                       // LOW-Byte aufteilen und einschreiben!
        I2C_send_2(0xb0,0x01,zw1,zw2);
        _wait_ms(10);                               // 10ms Abspeicherzeit, siehe Datenblatt
    
        printf("\n\n\n   Neue Frequenzeingabe per Tastendruck!\n");
        printf("===========================================");
        getchar();
      }
  }

