/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb der I2C-Bus Porterweiterung (PT - PCF8574)  */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     Zusatz Hardware:     PT - PCF8574                                        */
/*                                                                              */
/*                          Verbindung per 6-poliger MiniDIN-Leitung            */
/*                              Anschluss: X6 PT-FS  <->  X1 / X2 PT - PCF8574  */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     15.01.2010                                          */
/*                                                                              */
/*     Datei:               IO_PORTERWEITERUNG.c                                */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

#define PCF8574A_ADRESS  0x70           // Adresse des I2C-Bus Expander

/*** Definition der globalen Variablen  ***/
unsigned char bit SDA       @   0xc0;   // Portpin P4.0, I2C-Bus: serielle Datenleitung SDA
unsigned char bit SCL       @   0xc1;   // Portpin P4.1, I2C-Bus: serielle Taktleitung  SCL

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*  Funktion void set_cursor(unsigned char z, unsigned char s) positioniert den */
/*  Terminal Cursor auf die angegebene Position:                                */
/*  Parameter z := Zeile, Parameter s := Spalte                                 */
/********************************************************************************/

void set_cursor(unsigned char z, unsigned char s)
  {
    printf("\x1b\x59%c%c",(32+z),(32+s));
  }  

/********************************************************************************/
/*  Die Funktion void I2C_Start (void) erzeugt die (Repeated) Start-Bedingung   */
/*  des I2C-Busses!                                                             */
/********************************************************************************/

void I2C_Start (void)               
  {
    SDA = 1;
    SCL = 1;
    SDA = 0;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Stop (void) erzeugt die Stopp-Bedingung               */
/*  des I2C-Busses!                                                             */
/********************************************************************************/

void I2C_Stop (void)               
  {
    SDA = 0;
    SCL = 1;
    SDA = 1;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Sende(unsigned char Daten) sendet den Parameter Daten */
/*  seriell ber den I2C-Bus aus! Anschlieend wird das ACK/NACK-Signal vom     */
/*  Slave empfangen und ausgewertet!                                            */
/********************************************************************************/

void I2C_Sende(unsigned char Daten) 
  {
    unsigned char Maske=128;        // Hilfsvariable zum ausmaskieren des Sendebytes!
    unsigned char i;                // Schleifenvariable
    SCL = 0;                        // Serielle Taktleitung auf LOW
    for(i=0; i<8; i++)
      {
        if(Maske & Daten) SDA = 1;      // Wenn auszutaktendes Bit HIGH ist, SDA  
        else SDA = 0;                   // auf log. 1, ansonsten auf log. 0 setzen!
        SCL = 1;                        // Takt ausgeben und Bitmaske fr den
        Maske = Maske / 2;              // nchsten Schleifendurchlauf vorbereiten!
        SCL=0;
      }
    SDA = 1;                            // Ser. Datenleitung auf Eingang schalten!
    SCL = 1;                            // 9. Takt fr Acknowledgement des DS1621!
    if(SDA)                             // Wenn SDA log. 1, dann Fehlermeldung,
      {
        printf("\n\n\n Datenuebertragungsfehler:");
        printf("\n Kein Acknowledgement vom Slave, Programm angehalten !");
        while(1);
      }
    SCL = 0;                            // ansonsten weiter!
  }

/********************************************************************************/
/*  Die Funktion void I2C_Master_Ack (void) sendet das Acknowledgement Signal   */
/*  des Masters ber den I2C-Bus aus!                                           */
/********************************************************************************/

void I2C_Master_Ack (void)          
  {
    SDA = 0;
    SCL = 1;
    SCL = 0;
    SDA = 1;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Master_NAck (void) sendet das Not Acknowledgement     */
/*  Signal des Masters ber den I2C-Bus aus!                                    */
/********************************************************************************/

void I2C_Master_NAck (void)         
  {
    SDA = 1;
    SCL = 1;
    SCL = 0;
  }

/********************************************************************************/
/*  Die Funktion void I2C_send_light(unsigned char CB, unsigned char datum),    */
/*  sendet die Parameter CB (= Control Byte) und datum (= Datenbyte) seriell    */
/*  ber den I2C-Bus aus!                                                       */
/********************************************************************************/

void I2C_send_light(unsigned char CB, unsigned char datum)
  {    
    // Senden eines Bytes an den Salve
    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB);              // Sende Adresse des Slaves
    I2C_Sende(datum);           // Sende Byte an Slave
    I2C_Stop();                 // I2C-Bus Stopp Bedingung
  }

/********************************************************************************/
/*  Die Funktion unsigned char I2C_Empf(void) empfngt 1 Datenyte seriell ber  */
/*  I2C-Bus und liefert dieses als Rckgabewert der Funktion!                   */
/********************************************************************************/

unsigned char I2C_Empf(void)    
  {
    unsigned char Ergebnis=0;   // Variable fr das zu empfangene Datenbyte!
    unsigned char Wert=128;     // Hilfsvariable zum aufaddieren des Ergebnisses!
    unsigned char i;            // Schleifenvariable
    SCL = 0;                    // Serielle Taktleitung auf LOW
    SDA = 1;                    // Serielle Datenleitung auf Eingang schalten!
    for(i=0; i<8; i++)
      {
        SCL = 1;                // Takt ausgeben, anschlieend Zustand auf SDA 
        if(SDA) Ergebnis = Ergebnis + Wert; // prfen und wenn log. 1 aufaddieren!
        SCL=0;
        Wert = Wert / 2;        // Aufzuaddierenden Wert fr nchsten 
      }                         // Schleifendurchlauf vorbereiten!
    SDA = 1;                    // Ser. Datenleitung auf Eingang schalten!
    return(Ergebnis);           // Funktionsergebnis zurckgeben!
  }

/********************************************************************************/
/*  Die Funktionunsigned int I2C_empf_light(unsigned char CB) sendet den        */
/*  Parameter CB+1 (= Control Byte, anschlieend Datenrichtung Master liesst)   */
/*  seriell ber den I2C-Bus aus! Anschlieend wird 1 Datenbyte empfangen und   */
/* als Rckgabewert der Funktion zur Verfgung gestellt!                        */
/********************************************************************************/

unsigned int I2C_empf_light(unsigned char CB)
  {
    unsigned char daten;        // Datenbyte

    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB+1);            // Sende Adresse des Slaves: Lesen
    daten = I2C_Empf();         // Einlesen des Datenytes aus dem Slave
    I2C_Master_NAck();          // NOT Acknowlegdement Signal vom CC03er
    I2C_Stop();                 // I2C-Bus Stopp Bedingung
    return(daten);              // Rckgabe des zusammengesetzten Wertes
  }

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main(void)
  {
    unsigned char DATEN, RELAIS;                            // Hilfsvariable

    seri_init();                                            // Initialisierung der ser. Schnittstelle
    clear_screen();                                         // Bildschirm lschen

    printf(" Betrieb des I/O-Expanders! \n");               // Textmeldungen und Bildschirmmaske auf Hyperterm
    printf("============================\n\n");
    printf("   Programm laeuft ... ");
    set_cursor(9,3);
    printf("Eingaenge:\n\n");
    printf("   Werden gegen GND geschaltet, LED leuchtet bei logisch 0-Signal!");
    set_cursor(14,3);
    printf("Relais:\n\n");
    printf("   Sind bei logisch 0-Signal aktiv,\n");
    printf("   Relais-Kontakt ist geschlossen und LED leuchtet!");

    I2C_send_light(PCF8574A_ADRESS, 0xff);                  // Alle Relais inaktiv!
    
    while(1)                                                // Endlosschleife!      
      {
        DATEN = I2C_empf_light(PCF8574A_ADRESS);                // Zustand der 4 Eingnge (P2-P5) einlesen!          
        DATEN = DATEN >> 2;                                     // Zustand der vier Eingnge rechtsbndig
        DATEN = DATEN & 0x0f;                                   // schieben und die oberen 4 Bit auf 0 setzen!
        set_cursor(9,15);                                       // Cursor fr nachfolgende Textausgabe positionieren!
        switch (DATEN)                                          // Abhngig vom Zustand der Schalter
          {
            case 15 :   printf(" P5=1 P4=1 P3=1 P2=1");         // Textausgaben auf HyperTerm,
                        set_cursor(14,15);                      
                        printf(" K3=1 K2=1 K1=1 K0=1");
                        RELAIS = 0xff; break;                   // und Zustand der Relais speichern!
            case 14:    printf(" P5=1 P4=1 P3=1 P2=0"); 
                        set_cursor(14,15);
                        printf(" K3=1 K2=1 K1=1 K0=0");
                        RELAIS = 0xfe; break;
            case 13:    printf(" P5=1 P4=1 P3=0 P2=1"); 
                        set_cursor(14,15);
                        printf(" K3=1 K2=1 K1=0 K0=1");
                        RELAIS = 0xfd; break;
            case 12:    printf(" P5=1 P4=1 P3=0 P2=0"); 
                        set_cursor(14,15);
                        printf(" K3=1 K2=1 K1=0 K0=0");
                        RELAIS = 0xfc; break;
            case 11:    printf(" P5=1 P4=0 P3=1 P2=1"); 
                        set_cursor(14,15);
                        printf(" K3=1 K2=0 K1=1 K0=1");
                        RELAIS = 0xbf; break;
            case 10:    printf(" P5=1 P4=0 P3=1 P2=0"); 
                        set_cursor(14,15);
                        printf(" K3=1 K2=0 K1=1 K0=0");
                        RELAIS = 0xbe; break;
            case 9 :    printf(" P5=1 P4=0 P3=0 P2=1"); 
                        set_cursor(14,15);
                        printf(" K3=1 K2=0 K1=0 K0=1");
                        RELAIS = 0xbd; break;
            case 8 :    printf(" P5=1 P4=0 P3=0 P2=0"); 
                        set_cursor(14,15);
                        printf(" K3=1 K2=0 K1=0 K0=0");
                        RELAIS = 0xbc; break;
        
            case 7 :    printf(" P5=0 P4=1 P3=1 P2=1"); 
                        set_cursor(14,15);
                        printf(" K3=0 K2=1 K1=1 K0=1");
                        RELAIS = 0x7f; break;
            case 6 :    printf(" P5=0 P4=1 P3=1 P2=0"); 
                        set_cursor(14,15);
                        printf(" K3=0 K2=1 K1=1 K0=0");
                        RELAIS = 0x7e; break;
            case 5 :    printf(" P5=0 P4=1 P3=0 P2=1"); 
                        set_cursor(14,15);
                        printf(" K3=0 K2=1 K1=0 K0=1");
                        RELAIS = 0x7d; break;
            case 4  :   printf(" P5=0 P4=1 P3=0 P2=0"); 
                        set_cursor(14,15);
                        printf(" K3=0 K2=1 K1=0 K0=0");
                        RELAIS = 0x7c; break;
            case 3  :   printf(" P5=0 P4=0 P3=1 P2=1"); 
                        set_cursor(14,15);
                        printf(" K3=0 K2=0 K1=1 K0=1");
                        RELAIS = 0x3f; break;
            case 2  :   printf(" P5=0 P4=0 P3=1 P2=0"); 
                        set_cursor(14,15);
                        printf(" K3=0 K2=0 K1=1 K0=0");
                        RELAIS = 0x3e; break;
            case 1  :   printf(" P5=0 P4=0 P3=0 P2=1"); 
                        set_cursor(14,15);
                        printf(" K3=0 K2=0 K1=0 K0=1");
                        RELAIS = 0x3d; break;
            case 0  :   printf(" P5=0 P4=0 P3=0 P2=0"); 
                        set_cursor(14,15);
                        printf(" K3=0 K2=0 K1=0 K0=0");
                        RELAIS = 0x3c; break;
          }
        I2C_send_light(PCF8574A_ADRESS, RELAIS);                // Relaiszustnde zum PCF8574 schreiben!
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
