/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb des 8-stelligen LED-Displays PT - 7S,       */
/*                          Anzeige der zwei 8-Bit Analogmesswerte!             */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     Zusatz Hardware:     PT - 7S                                             */
/*                                                                              */
/*                          Verbindung per 6-poliger MiniDIN-Leitung            */
/*                              Anschluss: X7 PT-FS  <->  X2 PT - 7S            */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     14.01.2010                                          */
/*                                                                              */
/*     Datei:               LED_DISPLAY.c                                       */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

/*** Definition der globalen Variablen  ***/
unsigned char bit DIN       @   0xb2;   // Portpin P3.2, SPI-Bus: serielle Datenleitung
unsigned char bit CLK       @   0xb5;   // Portpin P3.5, SPI-Bus: serielle Taktleitung
unsigned char bit LOAD      @   0xb4;   // Portpin P3.4, SPI-Bus: serielle Freigabeleitung

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*  Funktion void set_cursor(unsigned char z, unsigned char s) positioniert den */
/*  Terminal Cursor auf die angegebene Position:                                */
/*  Parameter z := Zeile, Parameter s := Spalte                                 */
/********************************************************************************/

void set_cursor(unsigned char z, unsigned char s)
  {
    printf("\x1b\x59%c%c",(32+z),(32+s));
  }  

/********************************************************************************/
/*  Die Funktion unsigned int mw_erf8(unsigned char kanal) dient zum Betrieb    */
/*  des ON-Chip A/D-Wandlers des AT89C51CC03. Der Funktion wird der Eingangs-   */
/*  kanal bergeben und sie liefert ein 8-Bit Wandlungsergebnis als Funktions-  */
/*  wert zurck.                                                                */
/*  Wichtig:        Diese Funktion setzt voraus, das die Grundinitialisierung   */
/*                  des ON-Chip A/D-Wandlers ausserhalb der Funktion, also      */
/*                  z. B. im Hauptprogramm erfolgt. Zur Grundinitialisierung    */
/*                  muss der A/D-Wandler eingeschaltet werden, also das         */
/*                  ADEN-Bit im ADCON SFR gesetz werden ( ADCON = 0x20) und     */
/*                  der zu wandelnde Eingangskanal im SFR ADCF als Analog-      */
/*                  eingang definiert werden (z. B. ADCF = 0xff ->              */
/*                  gesamter Port 1 als Analogeingang definiert!                */
/*  Parameter kanal := Auswahl des A/D-Wandler Eingangskanals (0..7)            */
/********************************************************************************/

unsigned char mw_erf8(unsigned char kanal)
  {      
    ADCON = (ADCON & 0xe8);                 // Kanalauswahl und Wandlungsende-Bit rcksetzen!
    ADCON = (ADCON | (0x08+kanal));         // Kanalauswahl und Start der einmaligen Wandlung!
    while((ADCON & 0x10) == 0);             // Warte bis A/D-Wandlung beendet ist!
    return(ADDH);                           // Ergebnisregister auslesen und bergeben!
  }

/********************************************************************************/
/*  Die Funktion void send_spi_2(unsigned char adresse, unsigned char daten)    */
/*  sendet zwei Byte seriell vom C zur externen Peripherie-Einheit aus!        */
/*                                                                              */
/*  Die Parameter adresse und daten sind die auszusendenden Bytes,              */
/*  das MSB wird jeweils zuerst gesendet!                                       */
/*  Die Datenbernahme der externen Peripherieeinheit erfolgt bei ansteigender  */
/*  Taktflanke, das Freigabesignal ist aktiv LOW!                               */
/*                                                                              */
/*  Portleitungen:      P3.2 := Serielle Datenleitung (DIN)                     */
/*                      P3.5 := Serielle Taktleitung  (CLK)                     */
/*                      P3.4 := Freigabeleitung       (LOAD)                    */
/********************************************************************************/

void send_spi_2(unsigned char adresse, unsigned char daten)
  {
    unsigned char teiler, i;    // Hilfsvariablen zum bitweisen Austakten
    LOAD = 0;                   // Beginn der seriellen Datenbertragung
    teiler = 128;               // Bitmaske ( 1000 0000 ) fr das MSB
    for (i=1; i<9; i++)         // Schleife zum Austakten der 8 Adressbits
      {
        if(adresse & teiler) DIN = 1;   // DIN - Leitung bekommt den Logikpegel
        else DIN = 0;                   // des ausmaskierte, auszutaktenden Bits
        CLK = 1;                        // Takt ausgeben
        CLK = 0;
        teiler /= 2;                    // Bitmaske fr nchstes Adressbit setzen
      }
    teiler = 128;               // Bitmaske ( 1000 0000 ) fr das MSB
    for (i=1; i<9; i++)         // Schleife zum Austakten der 8 Datenbits
      {
        if(daten & teiler) DIN = 1;     // DIN - Leitung bekommt den Logikpegel
        else DIN = 0;                   // des ausmaskierte, auszutaktenden Bits
        CLK = 1;                        // Takt ausgeben
        CLK = 0;
        teiler /= 2;                    // Bitmaske fr nchstes Datenbit setzen
      }
    LOAD = 1;                   // Ende der seriellen Datenbertragung
  }

/********************************************************************************/
/*  Die Funktion void MAX7219_init(void) initialisiert den MAX7219 wie folgt:   */
/*  Code-B-Decode-Mode fr alle Displaystellen,                                 */
/*  alle Display-Stellen gelscht und mit maximaler Lichtintensitt aktiviert,  */
/*  1-sekndiger Displaytest!                                                   */
/********************************************************************************/

void MAX7219_init(void)
  {        
    unsigned char i;
    send_spi_2(0x09,0xff);      // Code-B-Decode-Mode fr alle Displaystellen gewhlt
    send_spi_2(0x0a,0x0f);      // Maximale Lichtintensitt gewhlt
    send_spi_2(0x0b,0x07);      // Alle 8 Display-Stellen aktiviert

    // Display lschen
    for (i=1; i<9; i++) send_spi_2(i,0x0f);

    send_spi_2(0x0c,0x01);      // Display ein
    send_spi_2(0x0f,0x01);      // Display-Test ein
    _wait_ms(1000);             // 1 sec warten
    send_spi_2(0x0f,0x00);      // Display-Test aus
  }

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main(void)
  {
    unsigned char wert, LED_H, LED_Z, LED_E;            // Hilfsvariable   

    seri_init();                                        // Initialisierung der ser. Schnittstelle
    clear_screen();                                     // Bildschirm lschen

    printf(" Betrieb des LED-Displays!\n");             // Textmeldungen und Bildschirmmaske auf Hyperterm
    printf("===========================\n\n");
    printf("   Messung von zwei analogen Eingangskanaele (Potis) im \n");
    printf("   8-Bit Modus undVisualisierung der Messwerte per PT-7S!\n\n");
    printf("   Programm laeuft ... \n\n\n\n\n");
    printf("   Messwert AN6:\n\n");
    printf("   Messwert AN7:");

    LOAD = 1;                                           // Grundzustand auf dem SPI-Bus einstellen!
    CLK = 0;
    MAX7219_init();                                     // MAX7219 initialisieren!

    ADCF = 0xc0;                                        // Port P1.6 und P1.7 als Analogeingang!
    ADCON = 0x20;                                       // ON-Chip A/D-Wandler eingeschaltet!

    while(1)                                            // Endlosschleife!      
      {
        wert = mw_erf8(6);                                  // Analogeingang AN6 erfassen
        set_cursor(11,20);
        printf("%3u", wert);

        LED_H = wert / 100;                                 // Wert fr Hunderterstelle berechnen,
        if (LED_H == 0) send_spi_2(0x03, 0x0f);             // wenn Wert Null ist, Blank sonst entsprechenden
        else send_spi_2(0x03, LED_H);                       // Wert auf LED-Displaystelle 3 ausgeben!

        LED_Z = (wert - (LED_H * 100)) / 10;                        // Wert fr Zehnerstelle berechnen, 
        if ((LED_Z == 0) && (LED_H == 0)) send_spi_2(0x02, 0x0f);   // und entsprechenden Wert auf
        else send_spi_2(0x02, LED_Z);                               // LED-Displaystelle 2 ausgeben!

        LED_E = (wert % 10);                                // Wert fr Temperatur Einerstelle
        send_spi_2(0x01, LED_E);                            // berechnen und auf LED-Displaystelle 1 ausgeben!

        wert = mw_erf8(7);                                  // Analogeingang AN7 erfassen
        set_cursor(13,20);
        printf("%3u", wert);

        LED_H = wert / 100;                                 // Wert fr Hunderterstelle berechnen,
        if (LED_H == 0) send_spi_2(0x07, 0x0f);             // wenn Wert Null ist, Blank sonst entsprechenden
        else send_spi_2(0x07, LED_H);                       // Wert auf LED-Displaystelle 7 ausgeben!

        LED_Z = (wert - (LED_H * 100)) / 10;                        // Wert fr Zehnerstelle berechnen, 
        if ((LED_Z == 0) && (LED_H == 0)) send_spi_2(0x06, 0x0f);   // und entsprechenden Wert auf
        else send_spi_2(0x06, LED_Z);                               // LED-Displaystelle 6 ausgeben!

        LED_E = (wert % 10);                                // Wert fr Temperatur Einerstelle
        send_spi_2(0x05, LED_E);                            // berechnen und auf LED-Displaystelle 5 ausgeben!
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
