/********************************************************************************/
/*                                                                              */
/*     Betrieb des PT - First Step!                                             */
/*                                                                              */
/*     Programm:            Betrieb des Temperatur-Sensors DS1621 (PT - DS1621) */
/*                                                                              */
/*     Autoren:             P.G. / B.v.B.                                       */
/*                                                                              */
/*     Zielsystem:          PT - First Step, Prozessor: AT89C51CC03 (Atmel)     */
/*                                                                              */
/*     Zusatz Hardware:     PT - DS1621                                         */
/*                                                                              */
/*                          Verbindung per 6-poliger MiniDIN-Leitung            */
/*                              Anschluss: X6 PT-FS  <->  X1 / X2 PT - DS1621   */
/*                                                                              */
/*     IDE/C-Compiler:      uC/51 (Wickenhuser)                                */
/*                                                                              */
/*     Letzte nderung:     14.01.2010                                          */
/*                                                                              */
/*     Datei:               TEMPERATUR.c                                        */
/*                                                                              */
/********************************************************************************/

/*** Einbinden von Include-Dateien ***/
#include <stdio.h>                      // Standard Ein-/Ausgabefunktionen
#include <at89C51cc03.h>                // CC03er-Grundregister

#define DS1621_ADRESS  0x90             // Adresse des Temperatur Chips 
#define READ_TEMP      0xaa             // Befehlswort Temperatur lesen
#define START_CONVERT  0xee             // Befehlswort Temperaturwandlung starten
#define STOP_CONVERT   0x22             // Befehlswort Temperaturwandlung beenden
#define ACCESS_TH      0xa1             // Befehlswort Temperatur Obergrenze
#define ACCESS_TL      0xa2             // Befehlswort Temperatur Untergrenze
#define ACCESS_CONFIG  0xac             // Befehlswort Konfigurationsregister

/*** Definition der globalen Variablen  ***/
unsigned char bit SDA       @   0xc0;   // Portpin P4.0, I2C-Bus: serielle Datenleitung SDA
unsigned char bit SCL       @   0xc1;   // Portpin P4.1, I2C-Bus: serielle Taktleitung  SCL

/********************************************************************************/
/*  Die Funktion void seri_init(void) initialisiert die ser. Schnittstelle des  */
/*  CC03ers bei einer Taktfrequenz von 11,0592MHz mit nachfolgenden Schnitt-    */
/*  stellen Parametern: 9600 Bd, 1 Stopp-Bit, keine Paritt, asynchr. Betrieb!  */
/********************************************************************************/

void seri_init(void)
  {
    SCON=0x52;
    TMOD |=0x20;
    TH1=0xfd;                      
    TR1=1;
    TI=1;
  }

/********************************************************************************/
/*  Funktion void clear_screen(void) lscht den Terminal Bildschirm!            */
/********************************************************************************/

void clear_screen(void)
  {
    printf("\x1b\x48\x1b\x4a");
  }

/********************************************************************************/
/*  Funktion void set_cursor(unsigned char z, unsigned char s) positioniert den */
/*  Terminal Cursor auf die angegebene Position:                                */
/*  Parameter z := Zeile, Parameter s := Spalte                                 */
/********************************************************************************/

void set_cursor(unsigned char z, unsigned char s)
  {
    printf("\x1b\x59%c%c",(32+z),(32+s));
  }  

/********************************************************************************/
/*  Die Funktion void I2C_Start (void) erzeugt die (Repeated) Start-Bedingung   */
/*  des I2C-Busses!                                                             */
/********************************************************************************/

void I2C_Start (void)               
  {
    SDA = 1;
    SCL = 1;
    SDA = 0;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Stop (void) erzeugt die Stopp-Bedingung               */
/*  des I2C-Busses!                                                             */
/********************************************************************************/

void I2C_Stop (void)               
  {
    SDA = 0;
    SCL = 1;
    SDA = 1;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Sende(unsigned char Daten) sendet den Parameter Daten */
/*  seriell ber den I2C-Bus aus! Anschlieend wird das ACK/NACK-Signal vom     */
/*  Slave empfangen und ausgewertet!                                            */
/********************************************************************************/

void I2C_Sende(unsigned char Daten) 
  {
    unsigned char Maske=128;        // Hilfsvariable zum ausmaskieren des Sendebytes!
    unsigned char i;                // Schleifenvariable
    SCL = 0;                        // Serielle Taktleitung auf LOW
    for(i=0; i<8; i++)
      {
        if(Maske & Daten) SDA = 1;      // Wenn auszutaktendes Bit HIGH ist, SDA  
        else SDA = 0;                   // auf log. 1, ansonsten auf log. 0 setzen!
        SCL = 1;                        // Takt ausgeben und Bitmaske fr den
        Maske = Maske / 2;              // nchsten Schleifendurchlauf vorbereiten!
        SCL=0;
      }
    SDA = 1;                            // Ser. Datenleitung auf Eingang schalten!
    SCL = 1;                            // 9. Takt fr Acknowledgement des DS1621!
    if(SDA)                             // Wenn SDA log. 1, dann Fehlermeldung,
      {
        printf("\n\n\n Datenuebertragungsfehler:");
        printf("\n Kein Acknowledgement vom Slave, Programm angehalten !");
        while(1);
      }
    SCL = 0;                            // ansonsten weiter!
  }

/********************************************************************************/
/*  Die Funktion void I2C_Master_Ack (void) sendet das Acknowledgement Signal   */
/*  des Masters ber den I2C-Bus aus!                                           */
/********************************************************************************/

void I2C_Master_Ack (void)          
  {
    SDA = 0;
    SCL = 1;
    SCL = 0;
    SDA = 1;
  }

/********************************************************************************/
/*  Die Funktion void I2C_Master_NAck (void) sendet das Not Acknowledgement     */
/*  Signal des Masters ber den I2C-Bus aus!                                    */
/********************************************************************************/

void I2C_Master_NAck (void)         
  {
    SDA = 1;
    SCL = 1;
    SCL = 0;
  }

/********************************************************************************/
/*  Die Funktion void I2C_send_cmd(unsigned char CB, unsigned char cmd) sendet  */
/*  die Parameter CB (= Control Byte) und cmd (= Commando Wort) seriell ber    */
/*  den I2C-Bus aus!                                                            */
/********************************************************************************/

void I2C_send_cmd(unsigned char CB, unsigned char cmd)
  {    
    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB);              // Sende Adresse des Slaves
    I2C_Sende(cmd);             // Sende Befehlswort an Salve
    I2C_Stop();                 // I2C-Bus Stopp Bedingung
  }

/********************************************************************************/
/*  Die Funktion void I2C_send_1(unsigned char CB, unsigned char cmd,           */
/*  unsigned char datum) sendet die Parameter CB (= Control Byte), cmd          */
/*  (= Commando Wort) und datum (= Datenbyte) seriell ber den I2C-Bus aus!     */
/********************************************************************************/

void I2C_send_1(unsigned char CB, unsigned char cmd, unsigned char datum)
  {    
    // Senden eines Bytes an den Salve
    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB);              // Sende Adresse des Slaves
    I2C_Sende(cmd);             // Sende Befehlswort an Salve
    I2C_Sende(datum);           // Sende Byte an Slave
    I2C_Stop();                 // I2C-Bus Stopp Bedingung
  }

/********************************************************************************/
/*  Die Funktion void I2C_send_1(unsigned char CB, unsigned char cmd,           */
/*  unsigned char dat1, unsigned char dat2) sendet die Parameter                */
/*  CB (= Control Byte), cmd (= Commando Wort), dat1 (= Datenbyte1) und         */
/*  dat2 (= Datenbyte2) seriell ber den I2C-Bus aus!                           */
/********************************************************************************/

void I2C_send_2(unsigned char CB, unsigned char cmd, unsigned char dat1, unsigned char dat2)
  {    
    // Senden der beiden Bytes an den Salve
    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB);              // Sende Adresse des Slaves
    I2C_Sende(cmd);             // Sende Befehlswort an Salve
    I2C_Sende(dat1);            // Sende 1. Byte an Slave
    I2C_Sende(dat2);            // Sende 2. Byte an Slave
    I2C_Stop();                 // I2C-Bus Stopp Bedingung
  }

/********************************************************************************/
/*  Die Funktion unsigned char I2C_Empf(void) empfngt 1 Datenyte seriell ber  */
/*  I2C-Bus und liefert dieses als Rckgabewert der Funktion!                   */
/********************************************************************************/

unsigned char I2C_Empf(void)    
  {
    unsigned char Ergebnis=0;   // Variable fr das zu empfangene Datenbyte!
    unsigned char Wert=128;     // Hilfsvariable zum aufaddieren des Ergebnisses!
    unsigned char i;            // Schleifenvariable
    SCL = 0;                    // Serielle Taktleitung auf LOW
    SDA = 1;                    // Serielle Datenleitung auf Eingang schalten!
    for(i=0; i<8; i++)
      {
        SCL = 1;                // Takt ausgeben, anschlieend Zustand auf SDA 
        if(SDA) Ergebnis = Ergebnis + Wert; // prfen und wenn log. 1 aufaddieren!
        SCL=0;
        Wert = Wert / 2;        // Aufzuaddierenden Wert fr nchsten 
      }                         // Schleifendurchlauf vorbereiten!
    SDA = 1;                    // Ser. Datenleitung auf Eingang schalten!
    return(Ergebnis);           // Funktionsergebnis zurckgeben!
  }

/********************************************************************************/
/*  Die Funktion unsigned int I2C_empf_2(unsigned char CB, unsigned char cmd)   */
/*  sendet die Parameter CB (= Control Byte) und cmd (= Commando Wort) seriell  */
/*  ber den I2C-Bus aus! Anschlieend werden 2 Datenbyte empfangen und als     */
/*  Rckgabewert der Funktion zur Verfgung gestellt!                           */
/********************************************************************************/

unsigned int I2C_empf_2(unsigned char CB, unsigned char cmd)
  {
    unsigned char HIGH;         // High-Byte
    unsigned char LOW;          // Low-Byte

    I2C_Start();                // I2C-Bus Start Bedingung
    I2C_Sende(CB);              // Sende Adresse des Slaves: Schreiben
    I2C_Sende(cmd);             // Sende Befehlswort an Slave

    I2C_Start();                // I2C-Bus Repeated Start Bedingung
    I2C_Sende(CB+1);            // Sende Adresse des Slaves: Lesen

    HIGH = I2C_Empf();          // Einlesen des HIGH-Bytes aus dem Slave
    I2C_Master_Ack();           // Acknowlegdement Signal vom CC03er

    LOW = I2C_Empf();           // Einlesen des LOW-Bytes aus dem Slave
    I2C_Master_NAck();          // NOT Acknowlegdement Signal vom CC03er

    I2C_Stop();                 // I2C-Bus Stopp Bedingung

    return(HIGH*256 + LOW);     // Rckgabe des zusammengesetzten Wertes
  }

/********************************************************************************/
/*  Die Funktion void DS1621_init (void) initialisiert den smarten Temperatur-  */
/*  sensor DS1621 wie folgt:                                                    */
/*                                                                              */
/*      -   Kontinuierliche Wandlung starten!                                   */
/*      -   Temperaturgrenzen fr Thermostatausgang:                            */
/*          Untere Temperaturgrenze -> 15 Grad Celsius (T_LOW)                  */
/*          Obere Temperaturgrenze  -> 30 Grad Celsius (T_HIGH)                 */
/*      -   Thermostatausgang aktiv LOW!                                        */
/********************************************************************************/

void DS1621_init(void)
  {
    signed char T_LOW = 15, T_HIGH = 30;                    // Hilfsvariablen, eingegebene Tempereaturgrenzen

    I2C_send_1(DS1621_ADRESS, ACCESS_CONFIG, 0x00);         // Kontinuierliche Wandlung, Ausgang aktiv LOW!
    I2C_send_2(DS1621_ADRESS, ACCESS_TL, T_LOW, 0x00);      // Untere Temperaturgrenze setzen!
    I2C_send_2(DS1621_ADRESS, ACCESS_TH, T_HIGH, 0x00);     // Obere Temperaturgrenze setzen!
    I2C_send_cmd(DS1621_ADRESS, START_CONVERT);             // Temperaturwandlung starten!
    printf ("\n\n\n   Initialisierung des DS1621 erfolgreich durchgefuehrt!");
  }

/********************************************************************************/
/*** Start des Hauptprogramms! **************************************************/
/********************************************************************************/

void main(void)
  {
    unsigned char Kommatemp;                                // Nachkommastelle des Temperaturmesswertes (0/5)
    signed char Ganztemp;                                   // Ganzzahliger Temperaturmesswert         
    unsigned int Temperatur;                                // Hilfsvariable 
    float Temp;                                             // Komplett Temperatur

    seri_init();                                            // Initialisierung der ser. Schnittstelle
    clear_screen();                                         // Bildschirm lschen

    printf(" Betrieb des Temperatur-Sensors DS1621!\n");    // Textmeldungen und Bildschirmmaske auf Hyperterm
    printf("========================================\n\n");
    printf("   Programm laeuft ... \n\n\n\n\n");
    printf("   Temperatur:       Grad Celsius");

    DS1621_init();                                          // Initialisierung des Temperatur-Sensors                    

    while(1)                                                // Endlosschleife!      
      {
        Temperatur = I2C_empf_2(DS1621_ADRESS, READ_TEMP);      // Temperaturmesswert einlesen
        Ganztemp = (Temperatur >> 8);                           // Ganzzahligen Temperaturwert ausfiltern
        Kommatemp = Temperatur;                                 // Nachkommawert der Temperatur ausfiltern
        if (Kommatemp) Temp = Ganztemp + 0.5;                   // Zusammenbau des gemessenen Temperaturwertes 
        else Temp = Ganztemp;

        set_cursor(8, 14);                                     // Cursor zur Ausgabe auf Hyperterm positionieren
        printf(" %5.1f", Temp);                                 // Temperaturausgabe auf Hyperterm
      }
  }

/*******************************************************************************************/
/*** Ende des Hauptprogramms, d.h. Ende des gesamten Programms! ****************************/
/*******************************************************************************************/
